$Title(MTV with software UART & HCS-II Protocol) 
$Date(08/05/92)
$MOD053
$include(\WGH\mtv.def)
$Debug

;**************************************************************************

VSYNC	EQU	0C7H
BAUDH	EQU	HIGH(0-94)	;timer value	
BAUDL	EQU	LOW(0-94)	;
STRTH	EQU	HIGH(0-16)	;one-half bit cell time
STRTL	EQU	LOW(0-16)	;
			
;CONSTANTS USED

XMTDAT	DATA	10H		;data for RS-232 transmit routine
RCVDAT	DATA	11H		;data for RS-232 receiver routine
BITCNT	DATA	12H		;transmit & receive bit count
TEMP1	DATA	13H
CHAR	DATA	14H
DATATR	DATA	15H
SPCATR	DATA	16H
NEWLAT	DATA	17H
CHKHEX	DATA	18H
HCOUNT	DATA	19H
OSMOD1  DATA	1AH
OSMOD2  DATA    1BH
QREGS	DATA	1CH		;d0-d3 for regs d4-d7 for netmode
QPORTS	DATA	1DH

FLAGS	DATA	20H

CHKERR	BIT	FLAGS.0
STRTOP	BIT	FLAGS.1
CECHO	BIT	FLAGS.2
TXFLAG	BIT	FLAGS.3		;transmit in progress flag
RXFLAG	BIT	FLAGS.4		;receive in progress flag
RXERR	BIT	FLAGS.5		;receiver framming error
RCVRDY	BIT	FLAGS.6		;receiver ready flag
 	
TXD	BIT	P3.4		;port bit for transmitter data
RXD	BIT	P3.3		;port bit for receiver data


PACKET	DATA	21H
NODEID	DATA	80H

;**************************************************************************

; Interrupt Vectors

	ORG	0		;Reset vector.
	LJMP	OSINIT

         ORG       03H                 ;External interrupt 0.
         LJMP      ISREX0              ;Indicates RS-232 start bit received.

         ORG       0BH                 ;Timer 0 interrupt.
         LJMP      Timr0               ;Baud rate generator.

;*****************************************************************************

OSINIT:	MOV	SP,#70h
	MOV	Flags,#0	;Clear RS-232 flags.
	MOV	TMOD,#01H	;16 bit timer 0
	MOV	TCON,#10h       ;Set up timer controls.
        MOV	IE,#82h         ;Enable timer 0 interrupts.
	MOV	QREGS,#10H	;set netmode to networks, query regs off
	MOV	QPORTS,#0	;set query ports off

;initialize the MTV parameters

	MOV	OSCON,#79H
	MOV	OSORG,#23H
	MOV	OSAD,#0
	MOV	OSMOD,#17H	;display off and shadow mode 7
	MOV	DATATR,#07H	;white foreground BK=video
	MOV	SPCATR,#04H	;blue background BK=video
	MOV	NEWLAT,#01	;normal with shadow on
	CALL	CLRSCRN		;init MTV OSD
	MOV	R0,#NODEID
	MOV	@R0,#'T'
	INC	R0
	MOV	@R0,#'V'
	INC	R0
	MOV	@R0,#'0'
	INC	R0
	MOV	@R0,#' '	;initial NODEID

;sync up to the beginning of a packet
;chksum packets begin with a '#'
;chksum-less packets begin with an '!'
;once the beginning of a packet is found store the entire
;packet in RAM until the end of the packet (i.e.- CR)
;is found. Also store the CR so that when the packet is scanned
;and interpreted, we will known where the packet ends.


MAIN:	CALL	ECHO
	CJNE	A,#'#',MAIN1	;if not '#' then try '!'
	MOV	R0,#PACKET	;start of message buffer
	JMP	SAVEMSG		;store message if it was a '#'
MAIN1:	CJNE	A,#'!',MAIN	;if not '!' then try again
	MOV	R0,#PACKET	;start of message buffer
SAVEMSG:MOV	@R0,A		;store the character
	INC	R0		;inx buffer pointer
	CALL	ECHO		;get next character
	CJNE	A,#0DH,SAVEMSG	;continue to get & save until CR
	MOV	@R0,A		;save the CR also
	MOV	R0,#PACKET	;point to first byte of packet
	MOV	A,@R0		;get first byte of packet
	CJNE	A,#'#',MAIN2	;if not '#' then no chksum so process packet
	CALL	VERIFY		;else try to verify chksum
	JB	CHKERR,MAIN	;ignore packet if checksum errors 
MAIN2:	CALL	PROCESS		;otherwise process the message
	JMP	MAIN		;and hunt for next message 


;***** PACKET PROCESS ROUTINE *****

PROCESS:MOV	R0,#PACKET	;start of message packet
PLOOP1:	MOV	A,@R0		;get byte from the packet
	CJNE	A,#0AH,CONV2UP	;if not return then covert to uppercase
	JMP	CHKPKT		;else check nodeid on packet
CONV2UP:CALL	UPCASE		;convert char to uppercase
	MOV	@R0,A		;store the char
	INC	R0		;point to next char
	JMP	PLOOP1		;and repeat
CHKPKT:	MOV	R0,#PACKET	;get first byte of packet and see
	CJNE	@R0,#'#',PNOCHK	;if this record has a checksum or not
	INC	R0		;and position the pointer to the NODEID
	INC	R0		;field of the packet. With a checksum
PNOCHK:	INC	R0		;NODEID = PACKET+4, without a checksum
	INC	R0		;NODEID = PACKET+2. Then check the NODEID.
	MOV	R1,#NODEID	;pointer to my NODEID
NODELP:	CJNE	@R1,#' ',NODECHR;stop checking on space - end of NODEID

	mov	dptr,#IDOK	;debug	
	CALL	PRINT		;debug
	call	crlf		;debug

	JMP	COMMAND		;NODEID is mine so process command string
NODECHR:MOV	A,@R0		;get NODEID char from received packet
	MOV	B,@R1		;get char from my NODEID
	CJNE	A,B,NOTME	;if ID fields don't match then its not me
	INC	R0		;otherwise the fields match so far and lets
	INC	R1		;continue to check the next character in 
	JMP	NODELP		;the ID fields
NOTME:
	mov	dptr,#IDFAIL	;debug
	CALL	print		;debug
	call	crlf		;debug

	RET			;not my NODEID ignore and wait for next packet


;***** COMMAND PROCESSOR ******

COMMAND:INC	R0		;set pointer to first char of command
	CJNE	@R0,#0DH,CTEST1	;if CR then its the end of the message 
	RET			;so wait for new message
CTEST1:	CJNE	@R0,#',',CTESTA	;if its a comma then get next command
	JMP	COMMAND 	;
CTESTA:	CJNE	@R0,#'A',CTESTF	;is it an 'A' ?
	JMP	ACMD		;
CTESTF:	CJNE	@R0,#'F',CTESTH	;is it an 'F' ?
	JMP	FCMD		;
CTESTH:	CJNE	@R0,#'H',CTESTL	;is it an 'H' ?
	JMP	HCMD		;
CTESTL:	CJNE	@R0,#'L',CTESTN	;is it an 'L' ?
	JMP	LCMD		;
CTESTN:	CJNE	@R0,#'N',CTESTP	;is it an 'N' ?
	JMP	NCMD		;
CTESTP:	CJNE	@R0,#'P',CTESTR	;is it an 'P' ?
	JMP	PCMD		;
CTESTR:	CJNE	@R0,#'R',CTESTS	;is it an 'R' ?
	JMP	RCMD		;
CTESTS:	CJNE	@R0,#'S',NXTCMA	;is it an 'S' ?
	INC	R0		;will get us around '=' sign
	JMP	SCMD		;
NXTCMA:	CJNE	@R0,#',',CTSTCR	;is it a COMMA ?
	JMP	COMMAND		;yes, continue command processing
CTSTCR:	CJNE	@R0,#0DH,CTSTL1	;is it a return ?
	RET			;yes, end of the message
CTSTL1:	INC	R0		;inc pointer to next character
	JMP	NXTCMA		;loop until comma or return


;****** VERIFY CHECKSUM ROUTINE *****
;
;	This routine performs two functions. First it reads the
;	checksum digits in the packet and stores them as HEX digits.
;	Second, the checksum digits are replaced with ASCII zeros and
;	the checksum of the string is calculated. If the checksum
;	matches, then the CHKERR flag is cleared, otherwise it is
;	set. Also, the checksum calculated on the string is 
;	converted to ASCII and stuffed into the checksum digits
;	position in the string. A received string in the buffer 
;	starting at location "PACKET" can be checked for checksum
;	errors by calling this routine and examining the CHKERR flag
;	on return. A string can be readied for transmission by storing
;	the string starting at the location "PACKET" with the checksum
;	digits containg ASCII zeroes and calling this routine.


VERIFY:	MOV	R0,#PACKET+1	;first checksum digit
	CALL	GETHEX		;get next HEX two digits
	MOV	CHKHEX,A	;and save in CHKHEX
	MOV	R0,#PACKET+1	;and then...
	MOV	A,#'0'		;stuff
	MOV	@R0,A		;checksum digits
	INC	R0		;with 
	MOV	@R0,A		;ASCII zereos
	CLR	A		;
	MOV	R0,#PACKET	;start of packet
ADDIT:	ADD	A,@R0		;add into checksum
	INC	R0		;point to next location
	CJNE	@R0,#0DH,ADDIT	;repeat till end of packet
	MOV	B,A		;save result in B
	ADD	A,CHKHEX	;add to chksum value in message
	JZ	CHKOK		;okay if zero		
CHKBAD: SETB	CHKERR		;set flag if bad
	JMP	STUFMSG		;and continue our task
CHKOK:	CLR	CHKERR		;set okay status and continue	
STUFMSG:CLR	A		;
	CLR	C		;
	SUBB	A,B		;
	MOV	B,A		;
	SWAP	A		;
	ANL	A,#0FH		;
	CALL	HEX2A		;convert to hex
	MOV	R0,#PACKET+1	;
	MOV	@R0,A		;
	INC	R0		;
	MOV	A,B		;
	ANL	A,#0FH		;
	CALL	HEX2A		;convert to hex
	MOV	@R0,A		;
	RET			;and done


;***** ASCII to HEX routine *****
;
;This routine accepts an ASCII char in the ACC
;and converts it into the corresponding hex digit.
;The routine checks to see if the char is in the
;range of '0' through '9' or in the range of 'A'
;through 'F'. If not in either range then the ASCII
;char is not a valid hex entry from the operator
;and an error flag is returned true along with the
;origina ASCI cha returned i th ACC.

A2HEX:	MOV	TEMP1,A		;save original char
	CLR	CHKERR		;clear error flag
	CLR	C		;carry will affect testing
	SUBB	A,#30H		;test char < 30H
	JC	AHERR		;error if char < 30H
	SUBB	A,#0AH		;char between 0 and 9 ?
	JNC	AHHIGH		;no, try 'A' thru 'F'
	ADD	A,#0AH		;add what we subtracted
	RET
AHHIGH:	CLR	C		;carry affects testing
	SUBB	A,#07H		;
	JC	AHERR		;error if char 3AH to 40H
	SUBB	A,#6H		;test char > 46H
	JNC	AHERR		;error if char >46H
	ADD	A,#10H		;adjust to correct value
	RET
AHERR:	MOV	A,TEMP1		;restore original char
	SETB	CHKERR		;set range error flag
	RET


;Clear MTV OSD screen (i.e. - fill with end of display chars)

CLRSCRN:MOV	OSAD,#0		;set address to zero
	MOV	OSAT,SPCATR
	MOV	OSDT,#BSPACE
	MOV	OSAT,NEWLAT
	MOV	OSDT,#NEWL
	MOV	R2,#7EH
	MOV	OSAT,#10H
CLRLP1:	MOV	OSDT,#NEWL
	DJNZ	R2,CLRLP1
	MOV	OSAT,DATATR
	MOV	OSAD,#2
	RET


ECHO:	CALL	CI		;read the byte
	JNB	P3.0,EXECHO	;don'techo if P3.0 is low
	PUSH	ACC		;save input byte
	CALL	CO		;echo the byte
	POP	ACC
EXECHO:	RET

CRLF:	MOV	A,#0DH
	CALL	CO
 	MOV	A,#0AH
	CALL	CO
	RET

;***** FUNCTION COMMAND *****

FCMD:	INC	R0		;point to next char
	CJNE	@R0,#'0',FCT1	;is it a 0 ?	
	CALL	CLRSCRN		;and clear the screen
	JMP	NXTCMA
FCT1:	CJNE	@R0,#'1',FCT2	;is it a 1 ?
	CALL	DSPON		;turn on display
	JMP	NXTCMA
FCT2:	CJNE	@R0,#'2',FCT3	;is it a 2 ?
	CALL	DSPOFF		;turn off display
	JMP	NXTCMA
FCT3:	CJNE	@R0,#'3',FCT4	;is it a 3 ?
	CALL	CLRBARS		;show color bar patterns
	JMP	NXTCMA
FCT4:	CJNE	@R0,#'4',FCT5	;is it a 4 ?
	CALL	FTTB		;wipe on effect
	JMP	NXTCMA
FCT5:	CJNE	@R0,#'5',FCEXIT	;is it a 5 ?	
	CALL	EBTT		;wipe off effect
FCEXIT:	JMP	NXTCMA

;***** SET NETWORK ADDRESS COMMAND *****

ACMD:	MOV	R1,#NODEID	;this is where the new network adr goes	
	INC	R0		;point to next char
	CJNE	@R0,#'=',ACEXIT	;exit if not "=" sign
	MOV	R2,#10H		;counter for max length of NODEID
ACL1:	INC	R0		;point to next char
	CJNE	@R0,#',',ACTCR	;is it a comma ?
	JMP	ACEXIT		;exit if a comma
ACTCR:	CJNE	@R0,#0DH,ACL2	;is it a return ?
	JMP	ACEXIT		;exit ifa return
ACL2:	MOV	A,@R0		;get the char
	MOV	@R1,A		;store char in NODEID field
	INC	R1		;bump up destination pointer 
	DJNZ	R2,ACL1		;repeat for up to 16 characters	
ACEXIT:	MOV	@R1,#' '	;store space delimiter after new NODEID
	JMP	NXTCMA		;exit

;***** PORT WRITE & QUERY COMMAND *******

PCMD:	INC	R0		;should point to port number now
	MOV	A,@R0		;get the char
	CALL	A2HEX		;convert to hex
	MOV	R2,A		;temp store in R2
	JB	CHKERR,PCEXIT	;check for valid hex digit
	ADD	A,#(0-4)	;exit if port is not in the 
	JC	PCEXIT		;range of 0 thru 3
	INC	R0		;pointer to next char
	CJNE	@R0,#'=',PCREAD	;if char not "=" then its a read command
	INC	R0		;points to first MSD of port data	
	CALL	GETHEX		;get next hex number
	MOV	B,A		;temp store in B
	MOV	DPTR,#PCWTBL	;base address for port write JMP table	
	MOV	A,R2		;get the port number
	ADD	A,R2		;double the value for 2 byte JMPS
	JMP	@A+DPTR		;and branch based on port number
PCWTBL:	AJMP	PCW0		;Port 0
	AJMP	PCW1		;Port 1
	AJMP	PCW2		;Port 2
	AJMP	PCW3		;Port 3

PCW0:	MOV	P0,B		;write the byte to the port
	LJMP	PCEXIT		;and done
PCW1:	MOV	P1,B		;write the byte to the port
	LJMP	PCEXIT		;and done
PCW2:	MOV	P2,B		;write the byte to the port
	LJMP	PCEXIT		;and done
PCW3:	MOV	P3,B		;write the byte to the port
	LJMP	PCEXIT		;and done

PCREAD:	MOV	DPTR,#PCRTBL	;base address for port write JMP table	
	MOV	A,R2		;get the port number
	ADD	A,R2		;double the value for 2 byte JMPS
	JMP	@A+DPTR		;and branch based on port number
PCRTBL:	AJMP	PCR0		;Port 0
	AJMP	PCR1		;Port 1
	AJMP	PCR2		;Port 2
	AJMP	PCR3		;Port 3

PCR0:	MOV	A,QPORTS
	SETB	ACC.0
	MOV	QPORTS,A
	SJMP	PCEXIT

PCR1:	MOV	A,QPORTS
	SETB	ACC.1
	MOV	QPORTS,A
	SJMP	PCEXIT

PCR2:	MOV	A,QPORTS
	SETB	ACC.2
	MOV	QPORTS,A
	SJMP	PCEXIT

PCR3:	MOV	A,QPORTS
	SETB	ACC.3
	MOV	QPORTS,A
	SJMP	PCEXIT

PCEXIT:	JMP	NXTCMA


;***** REGISTER WRITE & QUERY COMMAND *******

RCMD:	INC	R0		;should point to port number now
	MOV	A,@R0		;get the char
	CALL	A2HEX		;convert to hex
	MOV	R2,A		;temp store in R2
	JB	CHKERR,RCEXV	;check for valid hex digit
	ADD	A,#(0-8)	;exit if port is not in the 
	JC	RCEXV		;range of 0 thru 7
	INC	R0		;pointer to next char
	CJNE	@R0,#'=',RCREAD	;if char not "=" then its a read command
	INC	R0		;points to first MSD of port data	
	CALL	GETHEX		;get next hex number
	MOV	B,A		;temp store in B
	MOV	DPTR,#RCWTBL	;base address for REG write JMP table	
	MOV	A,R2		;get the REG number
	ADD	A,R2		;double the value for 2 byte JMPS
	JMP	@A+DPTR		;and branch based on port number
RCWTBL:	AJMP	RCW0		;OSDT
	AJMP	RCW1		;OSAT
	AJMP	RCW2		;OSCON
	AJMP	RCW3		;OSORG
	AJMP	RCW4		;OSMOD
	AJMP	RCW5		;DEFAULT CHAR ATTRIBUTE
	AJMP	RCW6		;DEFAULT BACKGROUND SPACE ATTRIBUTE
	AJMP	RCW7		;DEFAULT NEWLINE ATTRIBUTE

RCEXV:	LJMP	RCEXIT

RCW0:	MOV	OSDT,B		;write the byte to OSDT
	LJMP	RCEXIT		;and done
RCW1:	MOV	OSAT,B		;write the byte to OSAT
	LJMP	RCEXIT		;and done
RCW2:	MOV	OSCON,B		;write the byte to OSCON
	LJMP	RCEXIT		;and done
RCW3:	MOV	OSORG,B		;write the byte to OSORG
	LJMP	RCEXIT		;and done
RCW4:	MOV	OSMOD,B		;write the byte to OSMOD
	LJMP	RCEXIT		;and done
RCW5:	MOV	DATATR,B	;write the byte to default char attribute
	LJMP	RCEXIT		;and done
RCW6:	MOV	SPCATR,B	;write the byte to default bspace attribute
	LJMP	RCEXIT		;and done
RCW7:	MOV	NEWLAT,B	;write the byte to default newline attribute
	LJMP	RCEXIT		;and done

RCREAD:	MOV	DPTR,#RCRTBL	;base address for reg read JMP table	
	MOV	A,R2		;get the reg number
	ADD	A,R2		;double the value for 2 byte JMPS
	JMP	@A+DPTR		;and branch based on reg number

RCRTBL:	AJMP	RCR0		;OSDT
	AJMP	RCR1		;OSAT
	AJMP	RCR2		;OSCON
	AJMP	RCR3		;OSORG
	AJMP	RCR4		;OSMOD
	AJMP	RCR5		;DEFAULT CHAR ATTRIBUTE
	AJMP	RCR6		;DEFAULT BACKGROUND SPACE ATTRIBUTE
	AJMP	RCR7		;DEFAULT NEWLINE ATTRIBUTE

RCR0:	MOV	A,QREGS
	SETB	ACC.0
	MOV	QREGS,A
	SJMP	RCEXIT	

RCR1:	MOV	A,QREGS
	SETB	ACC.1
	MOV	QREGS,A
	SJMP	RCEXIT	

RCR2:	MOV	A,QREGS
	SETB	ACC.2
	MOV	QREGS,A
	SJMP	RCEXIT	

RCR3:	MOV	A,QREGS
	SETB	ACC.3
	MOV	QREGS,A
	SJMP	RCEXIT	

RCR4:	MOV	A,QREGS
	SETB	ACC.4
	MOV	QREGS,A
	SJMP	RCEXIT	

RCR5:	MOV	A,QREGS
	SETB	ACC.5
	MOV	QREGS,A
	SJMP	RCEXIT	

RCR6:	MOV	A,QREGS
	SETB	ACC.6
	MOV	QREGS,A
	SJMP	RCEXIT	

RCR7:	MOV	A,QREGS
	SETB	ACC.7
	MOV	QREGS,A
	SJMP	RCEXIT	

RCEXIT:	JMP	NXTCMA

;***** SET PORT BIT HIGH COMMAND *******

HCMD:	INC	R0		;should point to port number
	MOV	A,@R0		;get the char
	CALL	A2HEX		;convert to hex
	MOV	R2,A		;temp store in R2
	JB	CHKERR,HCEXV	;check for valid hex digit
	ADD	A,#(0-4)	;exit if port is not in the 
	JC	HCEXV		;range of 0 thru 3
	INC	R0		;pointer to next char
	CJNE	@R0,#'.',HCEXV	;exit if char not "."
	INC	R0		;points to bit of port
	MOV	A,@R0		;get the char
	CALL	A2HEX		;convert to hex
	MOV	B,A		;temp store in B
	MOV	DPTR,#HCTBL	;base address for bit set JMP table	
	MOV	A,R2		;get the port number
	ADD	A,R2		;double the value for 2 byte JMPS
	JMP	@A+DPTR		;and branch based on port number
HCTBL:	AJMP	HCP0		;Port 0
	AJMP	HCP1		;Port 1
	AJMP	HCP2		;Port 2
	AJMP	HCP3		;Port 3

HCEXV:	LJMP	HCEXIT

HCP0:	MOV	DPTR,#HP0TBL
	MOV	A,B
	ADD	A,B
	JMP	@A+DPTR

HP0TBL:	AJMP	HP0B0
	AJMP	HP0B1
	AJMP	HP0B2
	AJMP	HP0B3
	AJMP	HP0B4
	AJMP	HP0B5
	AJMP	HP0B6
	AJMP	HP0B7
	
HP0B0:	SETB	P0.0
	LJMP	HCEXIT

HP0B1:	SETB	P0.1
	LJMP	HCEXIT

HP0B2:	SETB	P0.2
	LJMP	HCEXIT

HP0B3:	SETB	P0.3
	LJMP	HCEXIT

HP0B4:	SETB	P0.4
	LJMP	HCEXIT

HP0B5:	SETB	P0.5
	LJMP	HCEXIT

HP0B6:	SETB	P0.6
	LJMP	HCEXIT

HP0B7:	SETB	P0.7
	LJMP	HCEXIT

HCP1:	MOV	DPTR,#HP1TBL
	MOV	A,B
	ADD	A,B
	JMP	@A+DPTR

HP1TBL:	AJMP	HP1B0
	AJMP	HP1B1
	AJMP	HP1B2
	AJMP	HP1B3
	AJMP	HP1B4
	AJMP	HP1B5
	AJMP	HP1B6
	AJMP	HP1B7
	
HP1B0:	SETB	P1.0
	LJMP	HCEXIT

HP1B1:	SETB	P1.1
	LJMP	HCEXIT

HP1B2:	SETB	P1.2
	LJMP	HCEXIT

HP1B3:	SETB	P1.3
	LJMP	HCEXIT

HP1B4:	SETB	P1.4
	LJMP	HCEXIT

HP1B5:	SETB	P1.5
	LJMP	HCEXIT

HP1B6:	SETB	P1.6
	LJMP	HCEXIT

HP1B7:	SETB	P1.7
	LJMP	HCEXIT

HCP2:	MOV	DPTR,#HP2TBL
	MOV	A,B
	ADD	A,B
	JMP	@A+DPTR

HP2TBL:	AJMP	HP2B0
	AJMP	HP2B1
	AJMP	HP2B2
	AJMP	HP2B3
	AJMP	HP2B4
	AJMP	HP2B5
	AJMP	HP2B6
	AJMP	HP2B7
	
HP2B0:	SETB	P2.0
	LJMP	HCEXIT

HP2B1:	SETB	P2.1
	LJMP	HCEXIT

HP2B2:	SETB	P2.2
	LJMP	HCEXIT

HP2B3:	SETB	P2.3
	LJMP	HCEXIT

HP2B4:	SETB	P2.4
	LJMP	HCEXIT

HP2B5:	SETB	P2.5
	LJMP	HCEXIT

HP2B6:	SETB	P2.6
	LJMP	HCEXIT

HP2B7:	SETB	P2.7
	LJMP	HCEXIT

HCP3:	MOV	DPTR,#HP3TBL
	MOV	A,B
	ADD	A,B
	JMP	@A+DPTR

HP3TBL:	AJMP	HP3B0
	AJMP	HP3B1
	AJMP	HP3B2
	AJMP	HP3B3
	AJMP	HP3B4
	AJMP	HP3B5
	AJMP	HP3B6
	AJMP	HP3B7
	
HP3B0:
	LJMP	HCEXIT

HP3B1:	SETB	P3.1
	LJMP	HCEXIT

HP3B2:	SETB	P3.2
	LJMP	HCEXIT

HP3B3:	SETB	P3.3
	LJMP	HCEXIT

HP3B4:	SETB	P3.4
	LJMP	HCEXIT

HP3B5:	SETB	P3.5
	LJMP	HCEXIT

HP3B6:	SETB	P3.6
	LJMP	HCEXIT

HP3B7:	SETB	P3.7
	LJMP	HCEXIT

HCEXIT:	JMP	NXTCMA


;***** CLEAR PORT BIT LOW COMMAND *******

LCMD:	INC	R0		;should point to port number
	MOV	A,@R0		;get the char
	CALL	A2HEX		;convert to hex
	MOV	R2,A		;temp store in R2
	JB	CHKERR,LCEXV	;check for valid hex digit
	ADD	A,#(0-4)	;exit if port is not in the 
	JC	LCEXV		;range of 0 thru 3
	INC	R0		;pointer to next char
	CJNE	@R0,#'.',LCEXV	;exit if char not "."
	INC	R0		;points to bit of port
	MOV	A,@R0		;get the char
	CALL	A2HEX		;convert to hex
	MOV	B,A		;temp store in B
	MOV	DPTR,#LCTBL	;base address for bit set JMP table	
	MOV	A,R2		;get the port number
	ADD	A,R2		;double the value for 2 byte JMPS
	JMP	@A+DPTR		;and branch based on port number
LCTBL:	AJMP	LCP0		;Port 0
	AJMP	LCP1		;Port 1
	AJMP	LCP2		;Port 2
	AJMP	LCP3		;Port 3

LCEXV:	LJMP	LCEXIT

LCP0:	MOV	DPTR,#LP0TBL
	MOV	A,B
	ADD	A,B
	JMP	@A+DPTR

LP0TBL:	AJMP	LP0B0
	AJMP	LP0B1
	AJMP	LP0B2
	AJMP	LP0B3
	AJMP	LP0B4
	AJMP	LP0B5
	AJMP	LP0B6
	AJMP	LP0B7
	
LP0B0:	CLR	P0.0
	LJMP	LCEXIT

LP0B1:	CLR	P0.1
	LJMP	LCEXIT

LP0B2:	CLR	P0.2
	LJMP	LCEXIT

LP0B3:	CLR	P0.3
	LJMP	LCEXIT

LP0B4:	CLR	P0.4
	LJMP	LCEXIT

LP0B5:	CLR	P0.5
	LJMP	LCEXIT

LP0B6:	CLR	P0.6
	LJMP	LCEXIT

LP0B7:	CLR	P0.7
	LJMP	LCEXIT

LCP1:	MOV	DPTR,#LP1TBL
	MOV	A,B
	ADD	A,B
	JMP	@A+DPTR

LP1TBL:	AJMP	LP1B0
	AJMP	LP1B1
	AJMP	LP1B2
	AJMP	LP1B3
	AJMP	LP1B4
	AJMP	LP1B5
	AJMP	LP1B6
	AJMP	LP1B7
	
LP1B0:	CLR	P1.0
	LJMP	LCEXIT

LP1B1:	CLR	P1.1
	LJMP	LCEXIT

LP1B2:	CLR	P1.2
	LJMP	LCEXIT

LP1B3:	CLR	P1.3
	LJMP	LCEXIT

LP1B4:	CLR	P1.4
	LJMP	LCEXIT

LP1B5:	CLR	P1.5
	LJMP	LCEXIT

LP1B6:	CLR	P1.6
	LJMP	LCEXIT

LP1B7:	CLR	P1.7
	LJMP	LCEXIT

LCP2:	MOV	DPTR,#LP2TBL
	MOV	A,B
	ADD	A,B
	JMP	@A+DPTR

LP2TBL:	AJMP	LP2B0
	AJMP	LP2B1
	AJMP	LP2B2
	AJMP	LP2B3
	AJMP	LP2B4
	AJMP	LP2B5
	AJMP	LP2B6
	AJMP	LP2B7
	
LP2B0:	CLR	P2.0
	LJMP	LCEXIT

LP2B1:	CLR	P2.1
	LJMP	LCEXIT

LP2B2:	CLR	P2.2
	LJMP	LCEXIT

LP2B3:	CLR	P2.3
	LJMP	LCEXIT

LP2B4:	CLR	P2.4
	LJMP	LCEXIT

LP2B5:	CLR	P2.5
	LJMP	LCEXIT

LP2B6:	CLR	P2.6
	LJMP	LCEXIT

LP2B7:	CLR	P2.7
	LJMP	LCEXIT

LCP3:	MOV	DPTR,#LP3TBL
	MOV	A,B
	ADD	A,B
	JMP	@A+DPTR

LP3TBL:	AJMP	LP3B0
	AJMP	LP3B1
	AJMP	LP3B2
	AJMP	LP3B3
	AJMP	LP3B4
	AJMP	LP3B5
	AJMP	LP3B6
	AJMP	LP3B7
	
LP3B0:
	LJMP	LCEXIT

LP3B1:	CLR	P3.1
	LJMP	LCEXIT

LP3B2:	CLR	P3.2
	LJMP	LCEXIT

LP3B3:	CLR	P3.3
	LJMP	LCEXIT

LP3B4:	CLR	P3.4
	LJMP	LCEXIT

LP3B5:	CLR	P3.5
	LJMP	LCEXIT

LP3B6:	CLR	P3.6
	LJMP	LCEXIT

LP3B7:	CLR	P3.7
	LJMP	LCEXIT

LCEXIT:	JMP	NXTCMA


;***** SET NETWORK MODE COMMAND *******

NCMD:	INC	R0		;should point to net mode number
	MOV	A,@R0		;get the char
	CALL	A2HEX		;convert to hex
	MOV	R2,A		;temp store in R2
	JB	CHKERR,NCEXIT	;check for valid hex digit
	ADD	A,#(0-3)	;exit if mode is not in the 
	JC	NCEXIT		;range of 0 thru 3
	MOV	A,QPORTS	;get place to store net mode
	ANL	A,#0FH		;clear out bits for net mode 
	ORL	A,R2		;combine in bits for net mode
	MOV	QPORTS,A	;and save net mode & reg bits
NCEXIT:	JMP	NXTCMA

;***** PROCESS TEXT STRING COMMAND *****

SCXLV:	LJMP	SCXLAT

SCMD:	INC	R0		;point to next char
	CJNE	@R0,#',',SCTSTCR;is it a comma ?
	JMP	SCEXIT		;exit if comma	
SCTSTCR:CJNE	@R0,#0DH,SBSLSH	;is it a return ?
	JMP	SCEXIT		;exit if a return
SBSLSH:	CJNE	@R0,#'\',SCXLV	;is it a backslash
	INC	R0		;point to next char
	CJNE	@R0,#'B',SCTSTC	;is it a 'B' ?
	MOV	OSAT,SPCATR	;get bspace attribute
	MOV	OSDT,#BSPACE	;write a bspace to OSD
	MOV	OSAT,DATATR	;restore data attribute
	JMP	SCMD		;and repeat
SCTSTC:	CJNE	@R0,#'C',SCTSTE	;is it a comma ?
	MOV	OSAT,DATATR	;get data attribute
	MOV	OSDT,#1CH	;write a comma to OSD
	JMP	SCMD		;repeat 
SCTSTE:	CJNE	@R0,#'E',SCTSTN	;is it an E ?
	CALL	OSDEND		;set up end of display
	JMP	SCMD		;and repeat
SCTSTN:	CJNE	@R0,#'N',SCTSTS	;is it an 'N' ?
	MOV	OSAT,NEWLAT	;get newline attribute
	MOV	OSDT,#NEWL	;write newline to OSD
	MOV	OSAT,DATATR	;restore data attribute
	JMP	SCMD		;and repeat
SCTSTS:	CJNE	@R0,#'S',SCTSTW	;is it an 'S' ?
	MOV	OSAT,SPCATR	;get bspace attribute
	MOV	OSDT,#SBSPACE	;write a split background space to OSD
	MOV	OSAT,DATATR	;restore data attribute
	JMP	SCMD		;and repeat
SCTSTW:	CJNE	@R0,#'W',SCMD	;repeat if its not a 'W'
	INC	R0		;pointer to next char
	MOV	A,@R0		;get the char
	CALL	A2HEX		;convert to hex
	MOV	R2,A		;temp store in R2
	JB	CHKERR,SCEXIT	;check for valid hex digit
	ADD	A,#(0-8)	;exit if reg is not in the 
	JC	SCEXIT		;range of 0 thru 7
	INC	R0		;points to first MSD of REG data	
	CALL	GETHEX		;get next hex number
	MOV	B,A		;temp store in B
	MOV	DPTR,#SCWTBL	;base address for REG write JMP table	
	MOV	A,R2		;get the REG number
	ADD	A,R2		;double the value for 2 byte JMPS
	JMP	@A+DPTR		;and branch based on port number
SCWTBL:	AJMP	SCW0		;OSDT
	AJMP	SCW1		;OSAT
	AJMP	SCW2		;OSCON
	AJMP	SCW3		;OSORG
	AJMP	SCW4		;OSMOD
	AJMP	SCW5		;DEFAULT CHAR ATTRIBUTE
	AJMP	SCW6		;DEFAULT BACKGROUND SPACE ATTRIBUTE
	AJMP	SCW7		;DEFAULT NEWLINE ATTRIBUTE

SCW0:	MOV	OSDT,B		;write the byte to OSDT
	LJMP	SCMD		;and done
SCW1:	MOV	OSAT,B		;write the byte to OSAT
	LJMP	SCMD		;and done
SCW2:	MOV	OSCON,B		;write the byte to OSCON
	LJMP	SCMD		;and done
SCW3:	MOV	OSORG,B		;write the byte to OSORG
	LJMP	SCMD		;and done
SCW4:	MOV	OSMOD,B		;write the byte to OSMOD
	LJMP	SCMD		;and done
SCW5:	MOV	DATATR,B	;write the byte to default char attribute
	LJMP	SCMD		;and done
SCW6:	MOV	SPCATR,B	;write the byte to default bspace attribute
	LJMP	SCMD		;and done
SCW7:	MOV	NEWLAT,B	;write the byte to default newline attribute
	LJMP	SCMD		;and done

SCXLAT:	MOV	A,@R0		;get the char
	CALL	XLAT		;translate char from ASCII to MTV
	MOV	OSAT,DATATR	;data attribute to OSAT
	MOV	OSDT,A		;update OSD
	JMP	SCMD		;repeat for next character

SCEXIT:	JMP	NXTCMA

;****** GET NEXT HEX NUMBER IN INPUT STREAM
;
;R0 must point to first digit of two digit hex number

GETHEX:	MOV	A,@R0		;get it
	CALL	A2HEX		;convert from ASCII to HEX
	SWAP	A		;move to high nibble
	MOV	B,A		;save in B register for now
	INC	R0		;point to second checksum digit	
	MOV	A,@R0		;get it
	CALL	A2HEX		;convert from ASCII to HEX
	ORL	A,B		;combine together
	RET			;and done

DSPON:	MOV	A,OSMOD	;get current mode
	SETB	ACC.5	;complement
	CLR	ACC.4	;mode bits
	MOV	OSMOD,A	;update mode
	RET

DSPOFF:	MOV	A,OSMOD	;get current mode
	CLR	ACC.5	;complement
	SETB	ACC.4	;mode bits
	MOV	OSMOD,A	;update mode
	RET

SHADTOG:MOV	A,NEWLAT
	CPL	ACC.0
	MOV	NEWLAT,A
	RET

SHADMOD:CALL	ECHO		;get & echo character
	ANL	A,#07H		;save only shadow mode bits
 	ANL	OSMOD,#0F8H	;clr bits 0 thru 2 in OSMOD
	ORL	OSMOD,A		;add in shadow mode bits
	RET			;and done

CLRBARS:CALL	DSPOFF		;turn off display
	CALL	CLRSCRN		;fill screen with ends
	MOV	OSORG,#2BH
	MOV	OSAT,#0
	MOV	OSDT,#NEWL
	MOV	OSAT,#07H	;white chars & video
	MOV	OSDT,#16H	;V
	MOV	OSDT,#09H	;I
	MOV	OSDT,#04H	;D
	MOV	OSDT,#05H	;E
	MOV	OSDT,#0FH	;O
	MOV	OSDT,#3CH	;NORM SP
	MOV	OSDT,#14H	;T
	MOV	OSDT,#05H	;E
	MOV	OSDT,#13H	;S
	MOV	OSDT,#14H	;T
	MOV	OSDT,#3CH	;NORM SP
	MOV	OSDT,#09H	;I
	MOV	OSDT,#0DH	;M
	MOV	OSDT,#01H	;A
	MOV	OSDT,#07H	;G
	MOV	OSDT,#05H	;E
	MOV	OSAT,#02H	;NEWL ATR ********
	MOV	OSDT,#NEWL	;NEWL
	MOV	OSAT,#17H	;white
	MOV	OSDT,#BSPACE
	MOV	OSAT,#13H	;YELLOW
	MOV	OSDT,#BSPACE
	MOV	OSAT,#16H	;CYAN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#12H	;GREEN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#15H	;MAGENTA
	MOV	OSDT,#BSPACE
	MOV	OSAT,#11H	;RED
	MOV	OSDT,#BSPACE
	MOV	OSAT,#14H	;BLUE
	MOV	OSDT,#BSPACE
	MOV	OSAT,#10H	;BLACK
	MOV	OSDT,#BSPACE
	MOV	OSAT,#02H	;NEWL ATR ********
	MOV	OSDT,#NEWL	;NEWL
	MOV	OSAT,#17H	;white
	MOV	OSDT,#BSPACE
	MOV	OSAT,#13H	;YELLOW
	MOV	OSDT,#BSPACE
	MOV	OSAT,#16H	;CYAN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#12H	;GREEN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#15H	;MAGENTA
	MOV	OSDT,#BSPACE
	MOV	OSAT,#11H	;RED
	MOV	OSDT,#BSPACE
	MOV	OSAT,#14H	;BLUE
	MOV	OSDT,#BSPACE
	MOV	OSAT,#10H	;BLACK
	MOV	OSDT,#BSPACE
	MOV	OSAT,#02H	;NEWL ATR ********
	MOV	OSDT,#NEWL	;NEWL
	MOV	OSAT,#17H	;white
	MOV	OSDT,#BSPACE
	MOV	OSAT,#13H	;YELLOW
	MOV	OSDT,#BSPACE
	MOV	OSAT,#16H	;CYAN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#12H	;GREEN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#15H	;MAGENTA
	MOV	OSDT,#BSPACE
	MOV	OSAT,#11H	;RED
	MOV	OSDT,#BSPACE
	MOV	OSAT,#14H	;BLUE
	MOV	OSDT,#BSPACE
	MOV	OSAT,#10H	;BLACK
	MOV	OSDT,#BSPACE
	MOV	OSAT,#02H	;NEWL ATR ********
	MOV	OSDT,#NEWL	;NEWL
	MOV	OSAT,#17H	;white
	MOV	OSDT,#BSPACE
	MOV	OSAT,#13H	;YELLOW
	MOV	OSDT,#BSPACE
	MOV	OSAT,#16H	;CYAN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#12H	;GREEN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#15H	;MAGENTA
	MOV	OSDT,#BSPACE
	MOV	OSAT,#11H	;RED
	MOV	OSDT,#BSPACE
	MOV	OSAT,#14H	;BLUE
	MOV	OSDT,#BSPACE
	MOV	OSAT,#10H	;BLACK
	MOV	OSDT,#BSPACE
	MOV	OSAT,#02H	;NEWL ATR ********
	MOV	OSDT,#NEWL	;NEWL
	MOV	OSAT,#17H	;white
	MOV	OSDT,#BSPACE
	MOV	OSAT,#13H	;YELLOW
	MOV	OSDT,#BSPACE
	MOV	OSAT,#16H	;CYAN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#12H	;GREEN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#15H	;MAGENTA
	MOV	OSDT,#BSPACE
	MOV	OSAT,#11H	;RED
	MOV	OSDT,#BSPACE
	MOV	OSAT,#14H	;BLUE
	MOV	OSDT,#BSPACE
	MOV	OSAT,#10H	;BLACK
	MOV	OSDT,#BSPACE
	MOV	OSAT,#02H	;NEWL ATR ********
	MOV	OSDT,#NEWL	;NEWL
	MOV	OSAT,#17H	;white
	MOV	OSDT,#BSPACE
	MOV	OSAT,#13H	;YELLOW
	MOV	OSDT,#BSPACE
	MOV	OSAT,#16H	;CYAN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#12H	;GREEN
	MOV	OSDT,#BSPACE
	MOV	OSAT,#15H	;MAGENTA
	MOV	OSDT,#BSPACE
	MOV	OSAT,#11H	;RED
	MOV	OSDT,#BSPACE
	MOV	OSAT,#14H	;BLUE
	MOV	OSDT,#BSPACE
	MOV	OSAT,#10H	;BLACK
	MOV	OSDT,#BSPACE
	MOV	OSAT,#0
	MOV	OSDT,#NEWL
	MOV	OSAT,#10H
	MOV	OSDT,#NEWL	;END
	CALL	DSPON		;turn display back on
	RET


;message printing routine *****
;
;Prints a message starting at the address
;passed in the DPTR. Message terminates with
;the '$' (dollar sign) character.

PRINT:	CLR	A		;zero acc
	MOVC	A,@A+DPTR	;get the char
	CJNE	A,#'$',PRCHAR	;print char if not '$'
	RET			;ret if char = '$'
PRCHAR:	CALL	CO		;print the char
	INC	DPTR		;point to next char
	JMP	PRINT		;and test it

PRIBYT:	PUSH	ACC
	SWAP	A
	ANL	A,#0FH
	CALL	HEX2A
	CALL	CO
	POP	ACC
	ANL	A,#0FH
	CALL	HEX2A
	CALL	CO
	RET

;***** HEX to ASCII routine *****
;
;This routine receives a single hex digit
;(a four bit nibble) in the ACC and returns
;the equivilent ASCII char in the ACC.

HEX2A:	CLR	C		;carry affects the testing
	SUBB	A,#0AH		;test for range of 0-9, A-F
	JNC	HAHIGH		;no carry then A-F range
	ADD	A,#3AH		;add offset for 0-9 range
	RET
HAHIGH:	ADD	A,#41H		;add in offset for A-F range
	RET



FTTB:	MOV	A,OSMOD
	SETB	ACC.5
	CLR	ACC.4
	MOV	OSMOD1,A
	MOV	A,OSMOD
	CLR	ACC.5
	SETB	ACC.4
	MOV	OSMOD2,A
FTTB1:	SETB	STRTOP
	CALL	WIPE
	MOV	A,OSMOD
	SETB	ACC.5
	CLR	ACC.4
	MOV	OSMOD,A
	RET

FBTT:	MOV	A,OSMOD
	CLR	ACC.5
	SETB	ACC.4
	MOV	OSMOD1,A
	MOV	A,OSMOD
	SETB	ACC.5
	CLR	ACC.4
	MOV	OSMOD2,A
	CLR	STRTOP
	CALL	WIPE
	MOV	A,OSMOD
	SETB	ACC.5
	CLR	ACC.4
	MOV	OSMOD,A
	RET

ETTB:	MOV	A,OSMOD
	CLR	ACC.5
	SETB	ACC.4
	MOV	OSMOD1,A
	MOV	A,OSMOD
	SETB	ACC.5
	CLR	ACC.4
	MOV	OSMOD2,A
	SETB	STRTOP
	CALL	WIPE
	MOV	A,OSMOD
	CLR	ACC.5
	SETB	ACC.4
	MOV	OSMOD,A
	RET


EBTT:	MOV	A,OSMOD
	SETB	ACC.5
	CLR	ACC.4
	MOV	OSMOD1,A
	MOV	A,OSMOD
	CLR	ACC.5
	SETB	ACC.4
	MOV	OSMOD2,A
	CLR	STRTOP
	CALL	WIPE
	MOV	A,OSMOD
	CLR	ACC.5
	SETB	ACC.4
	MOV	OSMOD,A
	RET

WIPE:	MOV	IE,#0
	MOV	HCOUNT,#0FFH
	MOV	TMOD,#05H
WAITVS:	JNB	VSYNC,$
	CLR	VSYNC
	MOV	OSMOD,OSMOD1
	CLR	TR0
	MOV	A,HCOUNT
	JB	STRTOP,STOP	
	CPL	A
	INC	A
STOP:	MOV	TL0,A
	MOV	TH0,#0FFH
	SETB	TR0
WAITHS:	JNB	TF0,$
	CLR	TF0
	MOV	OSMOD,OSMOD2
	DJNZ	HCOUNT,WAITVS
	CLR	TR0
	MOV	TMOD,#01H
	MOV	TCON,#10H
	MOV	IE,#83H
	RET


XLAT:	INC	A		;bypass RET instruction
	db	83h		;MOVC	A,@A+PC --- look up value in table
	RET			;and done
	DB	0FH		;solid box, NUL
	DB	0FH		;solid box, SOH
	DB	0FH		;solid box, STX
	DB	0FH		;solid box, ETX
	DB	0FH		;solid box, EOT
	DB	0FH		;solid box, ENQ
	DB	0FH		;solid box, ACK
	DB	0FH		;solid box, BEL 
	DB	0FH		;solid box, BS
	DB	0FH		;solid box, HT
	DB	0FH		;solid box, LF
	DB	0FH		;solid box, VT
	DB	0FH		;solid box, FF
	DB	0FH		;solid box, CR
	DB	0FH		;solid box, SO
	DB	0FH		;solid box, SI
	DB	0FH		;solid box, DLE
	DB	0FH		;solid box, DC1
	DB	0FH		;solid box, DC2
	DB	0FH		;solid box, DC3
	DB	0FH		;solid box, DC4
	DB	0FH		;solid box, NAK
	DB	0FH		;solid box, SYN
	DB	0FH		;solid box, ETB
	DB	0FH		;solid box, CAN
	DB	0FH		;solid box, EM
	DB	0FH		;solid box, SUB
	DB	0FH		;solid box, ESC
	DB	0FH		;solid box, FS
	DB	0FH		;solid box, GS
	DB	0FH		;solid box, RS
	DB	0FH		;solid box, US
	DB	3CH		;SPACE
	DB	26H		; !
	DB	2FH		; "
	DB	28H		; #
	DB	29H		; $
	DB	2AH		; %
	DB	2CH		; &
	DB	2BH		;'
	DB	2EH		;(
	DB	25H		;)
	DB	2DH		;*
	DB	22H		;+ inbox
	DB	1CH		;,
	DB	1DH		;-
	DB	1EH		;.
	DB	1FH		;/
	DB	30H		;0
	DB	31H		;1
	DB	32H		;2
	DB	33H		;3
	DB	34H		;4
	DB	35H		;5
	DB	36H		;6
	DB	37H		;7
	DB	38H		;8
	DB	39H		;9
	DB	20H		;:
	DB	1BH		;semi colon
	DB	21H		;LEFT ARROW
	DB	3BH		;=
	DB	23H		;RIGHT ARROW
	DB	24H		;?
	DB	27H		;@
	DB	01H		;A
	DB	02H		;B
	DB	03H		;C
	DB	04H		;D
	DB	05H		;E
	DB	06H		;F
	DB	07H		;G
	DB	08H		;H
	DB	09H		;I
	DB	0AH		;J
	DB	0BH		;K
	DB	0CH		;L
	DB	0DH		;M
	DB	0EH		;N
	DB	0FH		;O
	DB	10H		;P
	DB	11H		;Q
	DB	12H		;R
	DB	13H		;S
	DB	14H		;T
	DB	15H		;U
	DB	16H		;V
	DB	17H		;W
	DB	18H		;X
	DB	19H		;Y
	DB	1AH		;Z
	DB	0FH		;solid box, [
	DB	0FH		;solid box, \
	DB	0FH		;solid box, ]
	DB	00H		;UP caret
	DB	0FH		;solid box, underline
	DB	0FH		;solid box, blank
	DB	01H		;A
	DB	02H		;B
	DB	03H		;C
	DB	04H		;D
	DB	05H		;E
	DB	06H		;F
	DB	07H		;G
	DB	08H		;H
	DB	09H		;I
	DB	0AH		;J
	DB	0BH		;K
	DB	0CH		;L
	DB	0DH		;M
	DB	0EH		;N
	DB	0FH		;O
	DB	10H		;P
	DB	11H		;Q
	DB	12H		;R
	DB	13H		;S
	DB	14H		;T
	DB	15H		;U
	DB	16H		;V
	DB	17H		;W
	DB	18H		;X
	DB	19H		;Y
	DB	1AH		;Z
	DB	1BH		;solid box, {
	DB	1CH		;solid box, vertical bar
	DB	0FH		;solid box, }
	DB	0FH		;solid box, tidle
	DB	0FH		;solid box, DEL

;***** UPPER CASE CONVERSION ROUTINE *****
;
;Receives a char in the ACC and converts it to uppercase if it is
;in lowercase. Char is returned in the ACC.

UPCASE:	JNB	ACC.6,UPCSEX	;exit if not 4x,5x, or 6x Hex
	CLR	ACC.5		;convert to upper case
UPCSEX:	RET			;and done
	

; Send a byte out RS-232 and wait for completion before returning.
; (use if there is nothing else to do while RS-232 is busy)

CO:	JB        RxFlag,$            ;Wait for receive complete.
         ACALL     RSXmt               ;Send ACC to RS-232 output.
         JB        TxFlag,$            ;Wait for transmit complete.
         RET


; Begin RS-232 transmit.

RSXmt:   MOV       XmtDat,A            ;Save data to be transmitted.
         MOV       BitCnt,#10          ;Set bit count.
         MOV       TH0,#BAUDH	       ;Set timer for baud rate.
         MOV       TL0,#BAUDL
         SETB      TR0                 ;Start timer.
         CLR       TxD                 ;Begin start bit.
         SETB      TxFlag              ;Set transmit-in-progress flag.
         RET


; Timer 0 timeout: RS-232 receive bit or transmit bit.

Timr0:   PUSH      ACC
         PUSH      PSW
	 MOV       TH0,#BAUDH		;Set timer reload for baud rate.
         MOV       TL0,#BAUDL
         JB        RxFlag,RxBit        ;Is this a receive timer interrupt?
         JB        TxFlag,TxBit        ;Is this a transmit timer interrupt?
T0Ex1:   CLR       TR0                 ;Stop timer.
T0Ex2:   POP       PSW
         POP       ACC
         RETI


; RS-232 transmit bit routine.

TxBit:   DJNZ      BitCnt,TxBusy       ;Decrement bit count, test for done.
         CLR       TxFlag              ;End of stop bit, release timer.
         SJMP      T0Ex1               ;Stop timer and exit.

TxBusy:  MOV       A,BitCnt            ;Get bit count.
         CJNE      A,#1,TxNext         ;Is this a stop bit?
         SETB      TxD                 ;Set stop bit.
         SJMP      T0Ex2               ;Exit.

TxNext:  MOV       A,XmtDat            ;Get data.
         RRC       A                   ;Advance to next bit.
         MOV       XmtDat,A
         MOV       TxD,C               ;Send data bit.
         SJMP      T0Ex2               ;Exit.


;Begin RS-232 receive (after external interrupt 0).

ISREX0:	 MOV       BitCnt,#10          ;Set receive bit count.
         MOV       TH0,#STRTH		;First timeout in HALF a bit time.
         MOV       TL0,#STRTL
         MOV       RcvDat,#0           ;Initialize received data to 0.
         CLR       EX0                 ;Disable external interrupt 0.
         CLR       RxErr               ;Clear error flag.
         SETB      TR0                  ;Start timer.
         SETB      RxFlag              ;Set receive-in-progress flag.
         RETI


; RS-232 receive bit routine.

RxBit:   DJNZ      BitCnt,RxBusy       ;Decrement bit count, test for stop.
         JB        RxD,RxBitEx         ;Valid stop bit?
RxBtErr: SETB      RxErr               ;Bad stop bit, tell mainline.
RxBitEx: CLR       RxFlag              ;Release timer for other purposes.
         SETB      EX0                 ;Re-enable external interrupt 0.
         SETB      RcvRdy              ;Tell mainline that a byte is ready.
         SJMP      T0Ex1               ;Stop timer and exit.

RxBusy:  MOV       A,BitCnt            ;Get bit count.
         CJNE      A,#9,RxNext         ;Is this a start bit?
         JB        RxD,RxBtErr         ;Valid start bit?


         SJMP      T0Ex2               ;Exit.

RxNext:  MOV       A,RcvDat            ;Get partial receive byte.
         MOV       C,RxD               ;Get receive pin value.


         RRC       A                   ;Shift in new bit.
         MOV       RcvDat,A            ;Save updated receive byte.
         SJMP      T0Ex2               ;Exit.

CI:	 SETB      EX0                 ;Enable interrupt 0 (RS-232 receive).
         JNB       RcvRdy,$            ;Wait for data available.
         CLR       RcvRdy
	MOV	A,RCVDAT
	 RET

IDOK:	DB	0AH,0DH,'ID OK','$'

IDFAIL:	DB	0AH,0DH,'ID FAIL','$'

         END
